@@ -10,13 +10,13 @@
 *
 * $Project: /Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2005/04/19 20:52:55 +0200 $
-* $Revision: 6 $
+* $Date: 2006/01/04 00:02:26 +0100 $
+* $Revision: 10 $
 * $Source: /cbc/pack.h $
 *
 ********************************************************************************
 *
-* Copyright (c) 2002-2005 Marcus Holland-Moritz. All rights reserved.
+* Copyright (c) 2002-2006 Marcus Holland-Moritz. All rights reserved.
 * This program is free software; you can redistribute it and/or modify
 * it under the same terms as Perl itself.
 *
@@ -33,31 +33,41 @@
 #include "ctlib/cttype.h"
 
 #include "cbc/cbc.h"
-#include "cbc/idl.h"
+
 
 /*===== DEFINES ==============================================================*/
 
 /* values passed between all packing/unpacking routines */
-#define pPACKARGS   pTHX_ const CBC *THIS, PackInfo *PACK
-#define aPACKARGS   aTHX_ THIS, PACK
+#define pPACKARGS   pTHX_ PackHandle PACK
+#define aPACKARGS   aTHX_ PACK
 
 
 /*===== TYPEDEFS =============================================================*/
 
-typedef struct {
-  Buffer        buf;
-  IDList        idl;
-  SV           *bufsv;
-  SV           *self;
-} PackInfo;
+typedef struct PackInfo * PackHandle;
 
 
 /*===== FUNCTION PROTOTYPES ==================================================*/
 
-#define pack_type CBC_pack_type
-void pack_type(pPACKARGS, const TypeSpec *pTS, const Declarator *pDecl, int dimension, SV *sv);
+#define pk_create CBC_pk_create
+PackHandle pk_create(const CBC *THIS, SV *self);
 
-#define unpack_type CBC_unpack_type
-SV *unpack_type(pPACKARGS, const TypeSpec *pTS, const Declarator *pDecl, int dimension);
+#define pk_set_type CBC_pk_set_type
+void pk_set_type(PackHandle hdl, const char *type);
+
+#define pk_set_buffer CBC_pk_set_buffer
+void pk_set_buffer(PackHandle hdl, SV *bufsv, char *buffer, unsigned long buflen);
+
+#define pk_set_buffer_pos CBC_pk_set_buffer_pos
+void pk_set_buffer_pos(PackHandle hdl, unsigned long pos);
+
+#define pk_delete CBC_pk_delete
+void pk_delete(PackHandle hdl);
+
+#define pk_pack CBC_pk_pack
+void pk_pack(pPACKARGS, const TypeSpec *pTS, const Declarator *pDecl, int dimension, SV *sv);
+
+#define pk_unpack CBC_pk_unpack
+SV *pk_unpack(pPACKARGS, const TypeSpec *pTS, const Declarator *pDecl, int dimension);
 
 #endif
